import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingUtilities;

/**
 * 
 * Klasse fr Notenverwaltung
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: 01.11.2012 UTC
 */
public class GradeManagement extends JDialog implements
      WindowListener, Observer
{
   public Controller controller = null;

   private JPanel GridPanel = new JPanel();

   public ArrayList<JComboBox> subjects = new ArrayList<JComboBox>();

   public ArrayList<CustomGrade1Field> grade1 = new ArrayList<CustomGrade1Field>();

   public ArrayList<CustomGrade2Field> grade2 = new ArrayList<CustomGrade2Field>();

   public ArrayList<CustomGrade3Field> grade3 = new ArrayList<CustomGrade3Field>();

   public ArrayList<JButton> closeButtons = new ArrayList<JButton>();

   private JLabel cpNoteLabel;

   private JLabel ndNoteLabel;

   /**
    * 
    * Konstruktor
    * 
    */
   public GradeManagement(JFrame owner)
   {
      super(owner, "Notenverwaltung");
      addWindowListener(this);
      setLayout(null);

      GridPanel.setLayout(new GridLayout(0, 5));
      for (int j = 0; j < 2; j++)
      {
         addButtons();
      }

      JScrollPane sp = new JScrollPane(GridPanel,
            ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
            ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
      sp.setBounds(10, 15, 565, 100);
      add(sp);

      JButton addButton = new JButton("+ Hinzufgen");
      addButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            SwingUtilities.invokeLater(new Runnable()
            {
               public void run()
               {
                  addButtons();

                  // Formular muss danach neu gezeichnet werden
                  repaint();
                  GridPanel.repaint();
               }
            });
         }
      });
      addButton.setBounds(50, 120, 400, 30);
      add(addButton);

      JLabel cpLabel = new JLabel("Summe erzielter CPs:");
      cpLabel.setBounds(10, 160, 150, 30);
      add(cpLabel);
      cpNoteLabel = new JLabel("___");
      cpNoteLabel.setBounds(140, 160, 30, 30);
      add(cpNoteLabel);

      JLabel ndLabel = new JLabel("bislang erzielter ND:");
      ndLabel.setBounds(10, 180, 150, 30);
      add(ndLabel);
      ndNoteLabel = new JLabel("___");
      ndNoteLabel.setBounds(140, 180, 100, 30);
      add(ndNoteLabel);

      JButton closeButton = new JButton("Schlieen");
      closeButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            SwingUtilities.invokeLater(new Runnable()
            {
               public void run()
               {
                  dispose();
               }
            });
         }
      });
      closeButton.setBounds(415, 220, 160, 30);
      add(closeButton);

      // Gre des Fensters setzen
      setMinimumSize(new Dimension(600, 300));
   }

   private void addButtons()
   {
      final JComboBox subject = new JComboBox();
      // Default-Eintrag fr "nicht ausgewhlt" hinzufgen
      subject.addItem(Constants.getSubjectDescription());
      /**
       * Fcher aus dem Datenmodell der Combobox hinzufgen
       */
      if (controller != null)
      {
         for (SubjectDetails sd : controller.dataModel
               .getSubjectDetails())
         {
            subject.addItem(sd.getSubject());
         }
      }
      // Default-Eintrag vorselektieren
      subject.setSelectedItem(Constants.getSubjectDescription());
      subject.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            SwingUtilities.invokeLater(new Runnable()
            {
               public void run()
               {
                  String temp = (String) subject.getSelectedItem();
                  if (!temp.equals(Constants.getSubjectDescription()))
                  {
                     for (int x = 0; x < subjects.size(); x++)
                     {
                        if (subjects.get(x) != subject)
                        {
                           String temp2 = (String) subjects.get(x)
                                 .getSelectedItem();
                           if (temp2.equals(temp))
                           {
                              JOptionPane
                                    .showMessageDialog(
                                          null,
                                          "Fr dieses Fach wurden bereits Noten gespeichert!",
                                          "Achtung",
                                          JOptionPane.ERROR_MESSAGE);
                              subject.setSelectedItem(Constants.getSubjectDescription());
                           }
                        }
                     }
                  }

               }
            });
         }
      });
      subjects.add(subject);

      final CustomGrade1Field note1 = new CustomGrade1Field(3);
      grade1.add(note1);

      final CustomGrade2Field note2 = new CustomGrade2Field(3);
      note2.setEnabled(false);
      grade2.add(note2);

      final CustomGrade3Field note3 = new CustomGrade3Field(3);
      note3.setEnabled(false);
      grade3.add(note3);

      note1.addKeyListener(new KeyListener()
      {
         @Override
         public void keyPressed(KeyEvent arg0)
         {
         }

         @Override
         public void keyReleased(KeyEvent arg0)
         {
            boolean enableNextField = false;
            if (note1.getText().length() > 0)
            {
               if (note1.getText().substring(0, 1).equals("."))
               {
                  note1.setText("");
               }
               else if (!note1.getText().trim().equals(""))
               {
                  enableNextField = (Integer.parseInt(note1.getText()
                        .substring(0, 1)) >= 5);
               }
            }
            note2.setEnabled(enableNextField);
            if (!enableNextField)
            {
               note2.setText(Constants.getGrade2Description());
            }
            note3.setEnabled(enableNextField);
            if (!enableNextField)
            {
               note3.setText(Constants.getGrade3Description());
            }

            /**
             * Daten ins Model bernehmen OHNE bereits auf Festplatte zu
             * speichern
             */
            controller.triggerConfirmDataFromGM1();

            /**
             * erzielte CPs und ND im Form aktualisieren
             */
            Integer temp = controller.getPassedCPs();
            cpNoteLabel.setText(temp.toString());

            Float temp2 = controller.getPassedND();
            ndNoteLabel.setText(Float.toString(temp2));

            // Formular muss danach neu gezeichnet werden
            validate();
            repaint();
         }

         @Override
         public void keyTyped(KeyEvent arg0)
         {
         }
      });

      note2.addKeyListener(new KeyListener()
      {
         @Override
         public void keyPressed(KeyEvent arg0)
         {
         }

         @Override
         public void keyReleased(KeyEvent arg0)
         {
            boolean enableNextField = false;
            if (note2.getText().length() > 0)
            {
               if (note2.getText().substring(0, 1).equals("."))
               {
                  note2.setText("");
               }
               else if (!note2.getText().trim().equals(""))
               {
                  enableNextField = (Integer.parseInt(note2.getText()
                        .substring(0, 1)) >= 5);
               }
            }
            note3.setEnabled(enableNextField);
            if (!enableNextField)
            {
               note3.setText(Constants.getGrade3Description());
            }

            /**
             * Daten ins Model bernehmen OHNE bereits auf Festplatte zu
             * speichern
             */
            controller.triggerConfirmDataFromGM1();

            /**
             * erzielte CPs und ND im Form aktualisieren
             */
            Integer temp = controller.getPassedCPs();
            cpNoteLabel.setText(temp.toString());

            Float temp2 = controller.getPassedND();
            ndNoteLabel.setText(Float.toString(temp2));

            // Formular muss danach neu gezeichnet werden
            validate();
            repaint();
         }

         @Override
         public void keyTyped(KeyEvent arg0)
         {
         }
      });

      note3.addKeyListener(new KeyListener()
      {
         @Override
         public void keyPressed(KeyEvent arg0)
         {
         }

         @Override
         public void keyReleased(KeyEvent arg0)
         {
            /**
             * Daten ins Model bernehmen OHNE bereits auf Festplatte zu
             * speichern
             */
            controller.triggerConfirmDataFromGM1();

            /**
             * erzielte CPs und ND im Form aktualisieren
             */
            Integer temp = controller.getPassedCPs();
            cpNoteLabel.setText(temp.toString());

            Float temp2 = controller.getPassedND();
            ndNoteLabel.setText(Float.toString(temp2));
         }

         @Override
         public void keyTyped(KeyEvent arg0)
         {
         }
      });

      final JButton cbButton = new JButton("X");
      cbButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(ActionEvent arg0)
         {
            SwingUtilities.invokeLater(new Runnable()
            {
               public void run()
               {
                  int position = closeButtons.indexOf(cbButton);
                  GridPanel.remove(subjects.get(position));
                  GridPanel.remove(grade1.get(position));
                  GridPanel.remove(grade2.get(position));
                  GridPanel.remove(grade3.get(position));
                  GridPanel.remove(closeButtons.get(position));

                  /**
                   * Eingabeflchen mssen nicht nur aus dem Grid, sondern
                   * auch aus der Arraylist gelscht werden
                   */
                  subjects.remove(position);
                  grade1.remove(position);
                  grade2.remove(position);
                  grade3.remove(position);
                  closeButtons.remove(position);

                  // Formular muss danach neu gezeichnet werden
                  validate();
                  repaint();
               }
            });
         }
      });
      closeButtons.add(cbButton);

      GridPanel.add(subjects.get(subjects.size() - 1));
      GridPanel.add(grade1.get(grade1.size() - 1));
      GridPanel.add(grade2.get(grade2.size() - 1));
      GridPanel.add(grade3.get(grade3.size() - 1));
      GridPanel.add(closeButtons.get(closeButtons.size() - 1));

      validate();
      repaint();
   }

   // Implementiere WindowListener
   /**
    * wird aufgerufen wenn X-Button gedrckt wurde
    */
   public void windowClosing(WindowEvent event)
   {
      closeForm();
   }

   /**
    * wird aufgerufen wenn ber dispose() geschlossen wurde
    */
   public void windowClosed(WindowEvent event)
   {
      closeForm();
   }

   /**
    * weitere Methoden siehe
    * http://openbook.galileodesign.de/javainsel5/javainsel15_001
    * .htm#Rxx747java15001040005391F040104
    */
   public void windowDeiconified(WindowEvent event)
   {
   }

   public void windowIconified(WindowEvent event)
   {
   }

   public void windowActivated(WindowEvent event)
   {
   }

   public void windowDeactivated(WindowEvent event)
   {
   }

   public void windowOpened(WindowEvent event)
   {
   }

   /**
    * 
    * Separate Funktion fr Schlieen des Fensters
    * 
    */
   private void closeForm()
   {
      // System.out.println(subjects.size());

      /**
       * Daten bernehmen UND auf Festplatte speichern
       */
      controller.triggerConfirmDataFromGM2();
   }

   // Observer-Pattern
   @Override
   public void update(Observable o, Object arg)
   {
      /**
       * Felder zurcksetzen, auch die Arraylisten
       */
      GridPanel.removeAll();
      subjects.clear();
      grade1.clear();
      grade2.clear();
      grade3.clear();
      closeButtons.clear();

      // System.out.println("SDf");

      /**
       * Daten in View bernehmen
       */
      for (Grades gd : controller.dataModel.getGrades())
      {
         addButtons();
         boolean enableNextField = false;

         SubjectDetails selected = gd.getSubject();
         subjects.get(subjects.size() - 1).setSelectedItem(
               selected.getSubject());

         CustomGrade1Field note1 = grade1.get(grade1.size() - 1);
         note1.setText(Float.toString(gd.getGrade1()));
         note1.setEnabled(true);
         CustomGrade2Field note2 = grade2.get(grade2.size() - 1);
         note2.setText(Float.toString(gd.getGrade2()));
         CustomGrade3Field note3 = grade3.get(grade3.size() - 1);
         note3.setText(Float.toString(gd.getGrade3()));

         enableNextField = (gd.getGrade1() >= 5);
         note2.setEnabled(enableNextField);

         enableNextField = (gd.getGrade2() >= 5);
         note3.setEnabled(enableNextField);
      }

      Integer temp = controller.getPassedCPs();
      cpNoteLabel.setText(temp.toString());      
      
      Float temp2 = controller.getPassedND();
      ndNoteLabel.setText(Float.toString(temp2));     
   }
}
