import java.awt.Dialog.ModalityType;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Observable;

import javax.swing.JComboBox;
import javax.swing.SwingUtilities;

/**
 * 
 * Steuert alles, informiert Views bers Observer-Pattern ber nderungen
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: 09.11.2012 UTC
 */
public class Controller extends Observable
{
   private Controller controller = null;

   public DataModel dataModel;

   private MainFrame viewMF = null;

   private SubjectManagement viewSM = null;

   private GradeManagement viewGM = null;

   private FinalGrades viewFG = null;

   private String dataFile = "c:\\temp\\data.txt";

   final static String lineSeparator = System
         .getProperty("line.separator");
   
   // Anzahl Nachkommastellen fr Notendurchschnittsberechnung festlegen
   // .00, also zwei Nachkommastellen
   NumberFormat numberFormat = new DecimalFormat("0.00");   

   /**
    * 
    * Konstruktor, erzeugt Hauptfenster.
    * 
    */
   public Controller()
   {
      controller = this;
      // abschneiden festlegen
      numberFormat.setRoundingMode(RoundingMode.DOWN);      

      /**
       * erzeuge Datenmodell.
       */
      dataModel = new DataModel();

      /**
       * setze Pfad fr Datendatei - Projektverzeichnis -> System-Konstanten
       * siehe hier:
       * http://docs.oracle.com/javase/tutorial/essential/environment
       * /sysprop.html
       */
      String tempPath = System.getProperty("user.home");
      tempPath = tempPath.replace("\\", "/");
      if (tempPath.substring(tempPath.length() - 1) != "/")
         tempPath += "/";
      tempPath += "data.txt";
      dataFile = tempPath;

      /**
       * erzeuge alle Fenster und zeige Hauptfenster an.
       */
      SwingUtilities.invokeLater(new Runnable()
      {
         public void run()
         {
            viewMF = new MainFrame();

            viewSM = new SubjectManagement(viewMF);
            viewSM.controller = controller;
            viewSM.setModalityType(ModalityType.DOCUMENT_MODAL);
            viewSM.setVisible(false);

            viewGM = new GradeManagement(viewMF);
            viewGM.controller = controller;
            viewGM.setModalityType(ModalityType.DOCUMENT_MODAL);
            viewGM.setVisible(false);

            viewFG = new FinalGrades(viewMF);
            viewFG.controller = controller;
            viewFG.setModalityType(ModalityType.DOCUMENT_MODAL);
            viewFG.setVisible(false);

            viewMF.viewSM = viewSM;
            viewMF.viewGM = viewGM;
            viewMF.viewFG = viewFG;

            controller.addObserver(viewSM);
            controller.addObserver(viewGM);
            controller.addObserver(viewFG);
            readDataModelFromFile();

            viewMF.pack();
            viewMF.setVisible(true);
         }
      });
   }

   /**
    * 
    * View lst Aktion im Controller aus, der Daten aus dem View zieht - Daten
    * werden ins Model bernommen und anschlieend auf Festplatte geschrieben
    * 
    */
   public void triggerConfirmDataFromSM()
   {
      saveDataFromSMViewToModel();
   }

   private void saveDataFromSMViewToModel()
   {
      /*
       * bentigen Indexposition auch fr andere Felder
       */
      int position = 0;
      int usedRows = 0;

      for (CustomSubjectField csf : viewSM.subjects)
      {
         // System.out.println(csf.getText());
         if (!csf.getText().trim()
               .equals(Constants.getSubjectDescription())
               && !csf.getText().trim().equals(""))
         {
            usedRows++;

            /**
             * Schaue nach ob Fach bereits im Datenmodell existiert - wenn ja
             * Detaildaten aktualisieren - wenn nein neues Fach hinzufgen
             */
            SubjectDetails sd = dataModel.getSubjectByName(csf
                  .getText());
            boolean found = (sd != null);

            int temp = 0;
            if (!viewSM.CP.get(position).getText().trim()
                  .equals(Constants.getCPDescription())
                  && !viewSM.CP.get(position).getText().trim()
                        .equals(""))
            {
               temp = Integer.parseInt(viewSM.CP.get(position)
                     .getText().trim());
            }

            if (found)
            {
               sd.setCP(temp);
               sd.setSubjectType1(viewSM.subjectType1.get(position)
                     .isSelected());
               sd.setSubjectType2(viewSM.subjectType2.get(position)
                     .isSelected());
            }
            else
            {
               dataModel.addSubjectDetails(csf.getText(), temp,
                     viewSM.subjectType1.get(position).isSelected(),
                     viewSM.subjectType2.get(position).isSelected());
            }
         }

         position++;
      }

      /**
       * wenn Eintrge aus dem View gelscht wurden auch im Model lschen
       */
      while (dataModel.getSubjectDetails().size() != usedRows)
      {
         int count = dataModel.getSubjectDetails().size();
         for (int x = 0; x < count; x++)
         {
            SubjectDetails sd = dataModel.getSubjectDetails().get(x);
            // System.out.println(sd.getSubject());
            boolean found = false;

            for (CustomSubjectField csf : viewSM.subjects)
            {
               if (csf.getText().trim()
                     .equals(sd.getSubject().trim()))
               {
                  found = true;
                  break;
               }
            }

            if (!found)
            {
               // System.out.println("nicht mehr gefunden: "
               // + sd.getSubject());
               dataModel.clearSubjectXAndGrades(sd.getSubject());
               ;
               break;
            }
         }
      }

      /**
       * Test-Datenausgabe
       */

      // System.out.println(dataModel.getSubjectDetails().size());
      // System.out.println("->" + dataModel.getGrades().size());
      /*
       * for (int x=0; x<dataModel.getSubjectDetails().size();x++) {
       * System.out
       * .println(dataModel.getSubjectDetails().get(x).getSubject());
       * System.out.println(" CP=" +
       * dataModel.getSubjectDetails().get(x).getCP()); }
       * System.out.println(" ");
       */

      saveDataModelToFile();
   }

   /**
    * 
    * View lst Aktion im Controller aus, der Daten aus dem View zieht - Daten
    * werden ins Model bernommen und anschlieend ggf. auf Festplatte
    * geschrieben
    * 
    */
   public void triggerConfirmDataFromGM1()
   {
      saveDataFromGMViewToModel();
   }

   public void triggerConfirmDataFromGM2()
   {
      saveDataFromGMViewToModel();
      saveDataModelToFile();
   }

   private void saveDataFromGMViewToModel()
   {
      /*
       * bentigen Indexposition auch fr andere Felder
       */
      int position = 0;
      int usedRows = 0;

      for (JComboBox jcb : viewGM.subjects)
      {
         // System.out.println(jcb.getText());
         if (!jcb.getSelectedItem().equals(
               Constants.getSubjectDescription())
               && !jcb.getSelectedItem().equals(""))
         {
            usedRows++;

            /**
             * Schaue nach ob Notenzuordnung bereits im Datenmodell existiert
             * - wenn ja aktualisieren - wenn nein neue Notenzuordnung
             * hinzufgen
             */

            Grades gd = dataModel.getGradesBySubjectName((String) jcb
                  .getSelectedItem());
            boolean found = (gd != null);

            float tempGD1 = 0;
            float tempGD2 = 0;
            float tempGD3 = 0;
            if (!viewGM.grade1.get(position).getText().trim()
                  .equals(Constants.getGrade1Description())
                  && !viewGM.grade1.get(position).getText().trim()
                        .equals(""))
            {
               tempGD1 = Float.parseFloat(viewGM.grade1.get(position)
                     .getText().trim());
            }
            if (!viewGM.grade2.get(position).getText().trim()
                  .equals(Constants.getGrade2Description())
                  && !viewGM.grade2.get(position).getText().trim()
                        .equals(""))
            {
               tempGD2 = Float.parseFloat(viewGM.grade2.get(position)
                     .getText().trim());
            }
            if (!viewGM.grade3.get(position).getText().trim()
                  .equals(Constants.getGrade3Description())
                  && !viewGM.grade3.get(position).getText().trim()
                        .equals(""))
            {
               tempGD3 = Float.parseFloat(viewGM.grade3.get(position)
                     .getText().trim());
            }

            if (found)
            {
               gd.setGrade1(tempGD1);
               gd.setGrade2(tempGD2);
               gd.setGrade3(tempGD3);
            }
            else
            {
               dataModel.addGrades(
                     dataModel.getSubjectByName((String) jcb
                           .getSelectedItem()), tempGD1, tempGD2,
                     tempGD3);
            }
         }

         position++;
      }

      /**
       * wenn Eintrge aus dem View gelscht wurden auch im Model lschen
       */
      while (dataModel.getGrades().size() != usedRows)
      {
         int count = dataModel.getGrades().size();
         for (int x = 0; x < count; x++)
         {
            Grades gd = dataModel.getGrades().get(x);
            // System.out.println(gd.getSubject());
            boolean found = false;

            for (JComboBox jcb : viewGM.subjects)
            {
               if (jcb.getSelectedItem().equals(
                     gd.getSubject().getSubject().trim()))
               {
                  found = true;
                  break;
               }
            }

            if (!found)
            {
               // System.out.println("nicht mehr gefunden: "
               // + sd.getSubject());
               dataModel.clearGradesX(gd);
               break;
            }
         }
      }

      /**
       * Test-Datenausgabe
       */
      // System.out.println("->" + dataModel.getGrades().size());

      /*
       * for (int x = 0; x < dataModel.getGrades().size(); x++) {
       * System.out.println(dataModel.getGrades().get(x).getSubject()
       * .getSubject()); System.out.println("- 1. N. " +
       * dataModel.getGrades().get(x).getGrade1());
       * System.out.println("- 2. N. " +
       * dataModel.getGrades().get(x).getGrade2());
       * System.out.println("- 3. N. " +
       * dataModel.getGrades().get(x).getGrade3()); } System.out.println(" ");
       */
   }

   /**
    * 
    * View lst Aktion im Controller aus, der Daten aus dem View zieht - Daten
    * werden ins Model bernommen und anschlieend ggf. auf Festplatte
    * geschrieben
    * 
    */
   public void triggerConfirmDataFromFG1()
   {
      saveDataFromFGViewToModel();
   }

   public void triggerConfirmDataFromFG2()
   {
      saveDataFromFGViewToModel();
      saveDataModelToFile();
   }

   private void saveDataFromFGViewToModel()
   {
      if (!viewFG.naNote.getText().trim().equals(""))
      {
         dataModel.naNote = Float.parseFloat(viewFG.naNote.getText()
               .trim());
      }
      else
      {
         dataModel.naNote = 0;
      }

      if (!viewFG.nkNote.getText().trim().equals(""))
      {
         dataModel.nkNote = Float.parseFloat(viewFG.nkNote.getText()
               .trim());
      }
      else
      {
         dataModel.nkNote = 0;
      }
   }

   /**
    * 
    * Schreibe Datenmodell auf Festplatte.
    * 
    */
   public void saveDataModelToFile()
   {
      Writer fileObject;
      try
      {
         fileObject = new OutputStreamWriter(new FileOutputStream(
               dataFile));

         // .toString gibt es nicht beim primitiven Typen int, nur bei Integer
         Integer temp1 = dataModel.getSubjectDetails().size();
         writeValues(fileObject, "subjectsCount", temp1.toString());
         Integer temp2 = dataModel.getGrades().size();
         writeValues(fileObject, "gradesCount", temp2.toString());

         // Schreiben der Fcher
         ArrayList<SubjectDetails> subjectData1 = dataModel
               .getSubjectDetails();
         for (SubjectDetails sd : subjectData1)
         {
            Integer temp = sd.getCP();
            writeValues(fileObject, sd.getSubject(),
                  temp.toString() + ";" + sd.getSubjectType1() + ";"
                        + sd.getSubjectType2());
         }

         // Schreiben der Notenzuordnungen
         ArrayList<Grades> subjectData2 = dataModel.getGrades();
         for (Grades sd : subjectData2)
         {
            writeValues(
                  fileObject,
                  sd.getSubject().getSubject(),
                  Float.toString((sd.getGrade1())) + ";"
                        + Float.toString((sd.getGrade2())) + ";"
                        + Float.toString((sd.getGrade3())));
         }

         // Schreiben der Abschlussnote
         writeValues(fileObject, "naNote",
               Float.toString(dataModel.naNote));
         // Schreiben der Note des Kolloquiums
         writeValues(fileObject, "nkNote",
               Float.toString(dataModel.nkNote));

         // nicht vergessen fileObject wieder zu schlieen
         fileObject.close();
      }
      catch (FileNotFoundException e)
      {
         e.printStackTrace();
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }

      /**
       * Views ber nderungen informieren
       */
      tellChangedData();
   }

   private void writeValues(Writer fileObject, String string1,
         String string2)
   {
      try
      {
         fileObject.write(string1 + "=" + string2 + lineSeparator);
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }
   }

   /**
    * 
    * Liest Datenmodell von Festplatte ein.
    * 
    */
   public void readDataModelFromFile()
   {
      File file = new File(dataFile);
      if (!file.exists())
         return ;
         
      dataModel.clearAll();
      Integer subjectsCount = 0;
      Integer gradesCount = 0;

      try
      {
         int x = 0;
         BufferedReader in = new BufferedReader(new FileReader(
               dataFile));
         String dataRow = null;
         while ((dataRow = in.readLine()) != null)
         {
            if (x == 0)
            {
               // erste Zeile enthlt Anzahl Fcher
               subjectsCount = Integer
                     .parseInt(dataRow.split("=")[1]);
            }
            else if (x == 1)
            {
               // zweite Zeile enthlt Anzahl Fcher-Notenzuordnungen
               gradesCount = Integer.parseInt(dataRow.split("=")[1]);
            }
            else if ((x - 2 + 1) <= subjectsCount)
            {
               // erst danach kommt das Datenmodell
               // hier speziell die Fcher
               String temp1 = dataRow.split("=")[0];
               String temp2 = dataRow.split("=")[1];
               // System.out.println(temp2);
               Integer temp3 = Integer.parseInt(temp2.split(";")[0]);
               // System.out.println(temp3);
               boolean temp4 = temp2.split(";")[1].equals("true");
               // System.out.println(temp4);
               boolean temp5 = temp2.split(";")[2].equals("true");
               // System.out.println(temp5);

               // bernehmen ins Datenmodell
               dataModel
                     .addSubjectDetails(temp1, temp3, temp4, temp5);
            }
            else if ((x - 2 - subjectsCount + 1) <= gradesCount)
            {
               // hier werden die Notenzuordnungen eingelesen
               String temp1 = dataRow.split("=")[0];
               String temp2 = dataRow.split("=")[1];

               float temp3 = Float.parseFloat(temp2.split(";")[0]);
               float temp4 = Float.parseFloat(temp2.split(";")[1]);
               float temp5 = Float.parseFloat(temp2.split(";")[2]);

               // bernehmen ins Datenmodell
               dataModel.addGrades(dataModel.getSubjectByName(temp1),
                     temp3, temp4, temp5);
            }
            else if ((x - 2 - subjectsCount - gradesCount) == 0)
            {
               // Abschlussnote einlesen
               String temp1 = dataRow.split("=")[0];
               String temp2 = dataRow.split("=")[1];

               float temp3 = Float.parseFloat(temp2);

               // bernehmen ins Datenmodell
               dataModel.naNote = temp3;
            }
            else if ((x - 2 - subjectsCount - gradesCount) == 1)
            {
               // Note Kolloquium einlesen
               String temp1 = dataRow.split("=")[0];
               String temp2 = dataRow.split("=")[1];

               float temp3 = Float.parseFloat(temp2);

               // bernehmen ins Datenmodell
               dataModel.nkNote = temp3;
            }

            x++;
         }
         // nicht vergessen fileObject wieder zu schlieen
         in.close();
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }

      /**
       * Views ber nderungen informieren
       */
      tellChangedData();
   }

   /**
    * 
    * Gibt Summe der CP-Punkte aller bereits bestandenen Fcher zurck
    * 
    * @return Summe CP
    */
   public int getPassedCPs()
   {
      int result = 0;
      for (Grades gd : dataModel.getGrades())
      {
         if (dataModel.isSubjectPassed(gd))
         {
            result += gd.getSubject().getCP();
         }
      }
      return result;
   }

   /**
    * 
    * Gibt Notendurchschnitt der bestandenen Fcher zurck - ohne
    * Abschlussarbeit
    * 
    * @return Summe CP
    */
   public float getPassedND()
   {
      float result = 0;

      float grade = 0;
      int cp = 0;
      int sumCP = 0;

      boolean found = false;

      /**
       * Berechnung geschieht folgendermaen: ((Note1 * CP) + (Note2 * CP) ...
       * ) / Summe-CP
       */
      for (Grades gd : dataModel.getGrades())
      {
         if (dataModel.isSubjectPassed(gd))
         {
            found = true;

            grade = 0;
            cp = 0;
            if (!(gd.getGrade3() < 1.0) && (gd.getGrade3() < 5.0))
            {
               grade = gd.getGrade3();
            }
            else if (!(gd.getGrade2() < 1.0)
                  && (gd.getGrade2() < 5.0))
            {
               grade = gd.getGrade2();
            }
            else if (!(gd.getGrade1() < 1.0)
                  && (gd.getGrade1() < 5.0))
            {
               grade = gd.getGrade1();
            }

            cp = gd.getSubject().getCP();
            sumCP += cp;

            result += (grade * cp);
         }
      }

      if (found && sumCP!=0)
      {
         result /= sumCP;
      }

      return getRoundedND(result);
   }

   /**
    * 
    * Gibt Notendurchschnitt der bestandenen Fcher zurck - mit
    * Abschlussarbeit
    * 
    * @return Summe CP
    */
   public float getPassedNDWithGraduation()
   {
      float result = 0;

      float grade = 0;
      int cp = 0;
      int sumCP = 0;

      boolean found = false;

      /**
       * Berechnung geschieht folgendermaen: ((Note1 * CP) + (Note2 * CP) ...
       * ) / Summe-CP
       */
      for (Grades gd : dataModel.getGrades())
      {
         if (dataModel.isSubjectPassed(gd))
         {
            found = true;

            grade = 0;
            cp = 0;
            if (!(gd.getGrade3() < 1.0) && (gd.getGrade3() < 5.0))
            {
               grade = gd.getGrade3();
            }
            else if (!(gd.getGrade2() < 1.0)
                  && (gd.getGrade2() < 5.0))
            {
               grade = gd.getGrade2();
            }
            else if (!(gd.getGrade1() < 1.0)
                  && (gd.getGrade1() < 5.0))
            {
               grade = gd.getGrade1();
            }

            cp = gd.getSubject().getCP();
            sumCP += cp;

            result += (grade * cp);
         }
      }

      if (!(dataModel.naNote < 1.0) && (dataModel.naNote < 5.0))
      {
         result += (dataModel.naNote * 12);
         sumCP += 12;
      }

      if (!(dataModel.nkNote < 1.0) && (dataModel.nkNote < 5.0))
      {
         result += (dataModel.nkNote * 3);
         sumCP += 3;
      }

      if (found && sumCP!=0)
      {
         result /= sumCP;
      }

      return getRoundedND(result);
   }
   
   /**
    * 
    * Fr Berechnung des Notendurchschnitts auf zwei Nachkommastellen 
    * abschneiden
    *
    * @param nd
    * @return nd
    */
   private float getRoundedND(float nd)   
   {
      // format-Methode gibt sprachspezfisch mit Komma formatiert zurck, 
      // parseFloat-Methode erwartet aber einen Punkt
      // siehe http://openbook.galileodesign.de/javainsel5/javainsel04_006.htm#Rxx747java04006040001591F0121C3
      return Float.parseFloat(numberFormat.format(nd).replace(",", "."));
   }

   /**
    * 
    * benachrichtige Views, dass Daten gendert wurden.
    * 
    */
   public void tellChangedData()
   {
      // setChanged() markiert das Objekt als sendebereit, sodass bei
      // notifyObservers() Meldungen gegeben werden.
      setChanged();
      notifyObservers();
   }
}
