import java.util.ArrayList;

/**
 * 
 * Datenmodell.
 * 
 * @author $Author: Thomas Kramer
 * @version $Revision: $, $Date: 09.11.2012 UTC
 */
public class DataModel
{
   // Fcher
   private ArrayList<SubjectDetails> subjects = null;

   // Fcher-Notenzuordnungen
   private ArrayList<Grades> grades = null;

   // Abschlussnote
   public float naNote = 0;

   // Note Kolloquium
   public float nkNote = 0;

   /**
    * 
    * Konstruktor
    * 
    */
   public DataModel()
   {
      subjects = new ArrayList<SubjectDetails>();
      grades = new ArrayList<Grades>();
   }

   /**
    * 
    * Nimmt Fcherdetails in eine interne Arraylist auf.
    * 
    * @param subject
    *           Fach
    * @param cp
    *           Anzahl Credit Points
    * @param subjectType1
    *           Pflichtfach true/false
    * @param subjectType2
    *           Wahlfach true/false
    */
   public void addSubjectDetails(String subject, int cp,
         boolean subjectType1, boolean subjectType2)
   {
      SubjectDetails sd = new SubjectDetails(subject, cp,
            subjectType1, subjectType2);
      subjects.add(sd);
   }

   /**
    * 
    * Lsche alle Fcher und Notenzuordnungen.
    * 
    */
   public void clearAll()
   {
      subjects.clear();
      grades.clear();
   }

   /**
    * 
    * Lsche ein bestimmtes Fach und alle seine Noten-Zuordnungen (maximum 1
    * im view mglich)
    * 
    * @param subject
    *           Fach
    */
   public void clearSubjectXAndGrades(String subject)
   {
      int count1 = subjects.size();
      for (int x = 0; x < count1; x++)
      {
         if (subjects.get(x).getSubject().trim()
               .equals(subject.trim()))
         {
            int count2 = grades.size();
            for (int y = 0; y < count2; y++)
            {
               if (grades.get(y).getSubject() == subjects.get(x))
               {
                  grades.remove(y);
                  break;
               }
            }

            subjects.remove(x);
            break;
         }
      }
   }

   /**
    * 
    * Gibt alle Fcher zurck, und zwar als Kopie.
    * 
    * @return
    */
   public ArrayList<SubjectDetails> getSubjectDetails()
   {
      return new ArrayList<SubjectDetails>(subjects);
   }

   /**
    * 
    * Gibt ein bestimmtes Fach-Objekt zurck, Namen mssen ja eindeutig sein.
    * 
    * @param subject
    *           Fach-Bezeichnung
    * @return Fach-Objekt
    */
   public SubjectDetails getSubjectByName(String subject)
   {
      SubjectDetails result = null;
      for (SubjectDetails subjectObject : subjects)
      {
         if (subjectObject.getSubject().trim().equals(subject.trim()))
         {
            result = subjectObject;
         }
      }
      return result;
   }

   /**
    * 
    * Nimmt Noten zu einem Fach in eine interne Arraylist auf.
    * 
    * @param subject
    *           Fach
    * @param grade1
    *           Note1
    * @param grade2
    *           Note2
    * @param grade3
    *           Note3
    */

   public void addGrades(SubjectDetails subject, float grade1,
         float grade2, float grade3)
   {
      Grades grade = new Grades(subject, grade1, grade2, grade3);
      grades.add(grade);
   }

   /**
    * 
    * Lsche alle Notenzuordnungen zu Fchern.
    * 
    */
   public void clearGrades()
   {
      grades.clear();
   }

   /**
    * 
    * Lsche bestimmte Notenzuordnung
    * 
    * @param Grades
    *           Notenzuordnung
    */
   public void clearGradesX(Grades gd)
   {
      grades.remove(gd);
   }

   /**
    * 
    * Gibt alle Notenzuordnungen zurck, und zwar als Kopie.
    * 
    * @return Notenzuordnungen
    */
   public ArrayList<Grades> getGrades()
   {
      return new ArrayList<Grades>(grades);
   }

   /**
    * 
    * Gibt ein bestimmtes Fach-Objekt zurck, Namen mssen ja eindeutig sein.
    * 
    * @param subject
    *           Fach-Bezeichnung
    * @return Fach-Objekt
    */
   public Grades getGradesBySubjectName(String subject)
   {
      Grades result = null;
      for (Grades gradeObject : grades)
      {
         if (gradeObject.getSubject().getSubject().trim()
               .equals(subject.trim()))
         {
            result = gradeObject;
         }
      }
      return result;
   }
   
   /**
    * 
    * Gibt zurck ob das Fach bereits bestanden ist.
    *
    * @param grades
    * @return true wenn bestanden
    */
   public boolean isSubjectPassed(Grades grades)
   {
      boolean result = false;
      boolean note1 = (!(grades.getGrade1() < 1.0)) && (grades.getGrade1() < 5.0);
      boolean note2 = (!(grades.getGrade2() < 1.0)) && (grades.getGrade2() < 5.0);
      boolean note3 = (!(grades.getGrade3() < 1.0)) && (grades.getGrade3() < 5.0);
      result = (note1 || note2 || note3);
      
      //System.out.println("1." + grades.getGrade1() + " 2." + grades.getGrade2() + " 3."+ grades.getGrade3());
      //System.out.println(note1 + " " + note2 + " " + note3);
      //System.out.println(result);
      return result;
   }
   
}
